@file:Suppress("FunctionName")
package com.example.todolistapp.ui

import android.util.Log
import androidx.compose.animation.animateColorAsState
import androidx.compose.animation.core.Spring
import androidx.compose.animation.core.animateDpAsState
import androidx.compose.animation.core.animateFloatAsState
import androidx.compose.animation.core.spring
import androidx.compose.animation.core.tween
import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.combinedClickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.KeyboardArrowDown
import androidx.compose.material.icons.twotone.Edit
import androidx.compose.material3.Checkbox
import androidx.compose.material3.Icon
import androidx.compose.material3.Surface
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import com.example.todolistapp.data.Task
import com.example.todolistapp.ui.theme.ToDoListAppTheme
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.draw.rotate
import com.example.todolistapp.data.DataSource
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextDecoration
import com.example.todolistapp.R
import com.example.todolistapp.database.ToDoRepository
import kotlinx.coroutines.launch

@OptIn(ExperimentalFoundationApi::class)
@Composable
fun TaskItem(
    task: Task,
    onEditClick: () -> Unit,
    onDelete: () -> Unit,
    onCheckedChange: (Boolean) -> Unit,
    modifier: Modifier = Modifier
) {
    var expanded by remember { mutableStateOf(false) } // Toggle expanded state
    val rotationAngle by animateFloatAsState(
        targetValue = if (expanded) 180f else 0f,
        animationSpec = tween(durationMillis = 500),
        label = "Chevron Rotation"
    )
    val contentHeight by animateDpAsState(
        targetValue = if (expanded) (3 * 56).dp else 56.dp,
        animationSpec = spring(
            stiffness = Spring.StiffnessLow,
            dampingRatio = Spring.DampingRatioMediumBouncy
        ),
        label = "Content Height Animation"
    )

    val surfaceColor by animateColorAsState(
        targetValue = if (task.isCompleted)
            MaterialTheme.colorScheme.secondary.copy(alpha = 0.75f) else MaterialTheme.colorScheme.surface,
        animationSpec = tween(durationMillis = 500),
    )
    Surface(
        modifier = modifier
            .fillMaxWidth()
            .padding(8.dp)
            .combinedClickable(
                onClick = { expanded = !expanded },
                onLongClick = { onDelete() }
            ),
        color = surfaceColor,
        shape = RoundedCornerShape(12.dp),
        shadowElevation = 4.dp
    ) {
        Column {
            Row(
                verticalAlignment = androidx.compose.ui.Alignment.CenterVertically,
                modifier = Modifier.fillMaxWidth()
            ) {
                Checkbox(
                    checked = task.isCompleted,
                    onCheckedChange = { onCheckedChange(it) }
                )
                Text(
                    text = task.name,
                    textDecoration = if (task.isCompleted)
                        TextDecoration.LineThrough else TextDecoration.None,
                    modifier = Modifier.weight(1f)
                )
                IconButton(onClick = onEditClick) {
                    Icon(Icons.TwoTone.Edit, contentDescription = "Edit Task")
                }
                Icon(
                    imageVector = Icons.Default.KeyboardArrowDown,
                    contentDescription = "Expand",
                    modifier = Modifier
                        .size(24.dp)
                        .rotate(rotationAngle)
                )
            }

            // Show task description when expanded
            if (expanded && task.description.isNotBlank()) {
                Text(
                    text = task.description,
                    style = MaterialTheme.typography.bodyMedium,
                    modifier = Modifier.padding(start = 16.dp, end = 16.dp, bottom = 8.dp)
                )
            }
        }
    }
}

@Composable
fun DeleteConfirmationDialog(
    task: Task,
    onConfirm: () -> Unit,
    onDismiss: () -> Unit
) {
    AlertDialog(
        onDismissRequest = onDismiss,
        title = { Text(stringResource(R.string.delete_task_title)) },
        text = { Text(stringResource(R.string.delete_msg, task.name)) },
        confirmButton = {
            TextButton(onClick = onConfirm) {
                Text(stringResource(R.string.delete), color = Color.Red)
            }
        },
        dismissButton = {
            TextButton(onClick = onDismiss) {
                Text(stringResource(R.string.cancel))
            }
        }
    )
}

@OptIn(ExperimentalFoundationApi::class)
@Composable
fun ToDoListScreen(repository: ToDoRepository, modifier: Modifier, onEdit: (Task) -> Unit = {},) {
    var showDeleteDialog by remember { mutableStateOf(false) } // State for dialog visibility
    var taskToDelete: Task? by remember { mutableStateOf(null) } // Track the task to delete
    val tasks by repository.tasksFlow.collectAsState(initial = emptyList())
    Log.d("ToDoListScreen", "Tasks: $tasks")
    val coroutineScope = rememberCoroutineScope()
    Surface(
        modifier = modifier.padding(8.dp)
    ) {
        Column {
            if (showDeleteDialog && taskToDelete != null) {
                DeleteConfirmationDialog(
                    task = taskToDelete!!,
                    onConfirm = {
                        coroutineScope.launch {
                            repository.deleteTask(taskToDelete!!)
                            taskToDelete = null
                        }
                        showDeleteDialog = false
                    },
                    onDismiss = {
                        taskToDelete = null
                        showDeleteDialog = false
                    }
                )
            }
            LazyColumn(
                modifier = Modifier
                    .padding(bottom = 8.dp)
            ) {
                items(
                    items = tasks,
                ) { task ->
                    TaskItem(
                        task = task,
                        onEditClick = { onEdit(task) },
                        onDelete = {
                            showDeleteDialog = true
                            taskToDelete = task
                        },
                        onCheckedChange = {
                            coroutineScope.launch {
                                repository.updateTask(task.copy(isCompleted = it))
                            }
                        },
                        modifier = Modifier.animateItem(
                            fadeInSpec = tween(2000),
                            fadeOutSpec = spring(
                                stiffness = Spring.StiffnessLow,
                                dampingRatio = Spring.DampingRatioMediumBouncy
                            ),
                            placementSpec = spring(
                                stiffness = Spring.StiffnessLow,
                                dampingRatio = Spring.DampingRatioMediumBouncy
                            )
                        )
                    )
                }
            }
        }
    }
}
//@Preview
//@Composable
//fun ToDoListScreenPreview() {
//    ToDoListAppTheme {
//        ToDoListScreen()
//    }
//}

//@Preview
//@Composable
//fun DeleteConfirmationDialogPreview() {
//    ToDoListAppTheme {
//        DeleteConfirmationDialog(
//            task = Task("Task 1", "Description of task 1"),
//            onConfirm = {},
//            onDismiss = {}
//        )
//    }
//}
